// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <be/interface/Alert.h>
#include <be/interface/MenuItem.h>
#include <be/interface/PopUpMenu.h>
#include <be/support/String.h>

#include <be/be_apps/Deskbar/Deskbar.h>

#include <posix/stdio.h>		// sprintf()

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "Version.h"
#include "WatchApp.h"			// APP_SIGNATURE
#include "WatchView.h"
#include "WebWatch.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

const rgb_color COLOR_BACKGROUND = {184, 191, 201};
const rgb_color COLOR_FOREGROUND = {  0,   0,   0};

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

WatchView::WatchView(void)
	: BView(BRect(0,0,1,1), 0, 0, 0)
{
	// Do nothing.
}

// ===========================================================================
// Constructor
// ===========================================================================

WatchView::WatchView(BMessage* message)
	: BView(BRect(0,0,15,15),
			"WebWatch",
			B_FOLLOW_NONE,
			B_WILL_DRAW | B_PULSE_NEEDED) 
{
	// Initialize attributes.
	oldInternetTime = 0;
	
	// Calculate the maximum width for our replicant. Note that the Deskbar
	// accepts replicants that are wider than 16 pixels, but not replicants
	// that are higher than 16 pixels.
	float width = StringWidth("@999") + 4;
	ResizeTo(width, 15);

	// Set our view's background color.
	SetViewColor(COLOR_BACKGROUND);
}

// ===========================================================================
// Instantiate
// ===========================================================================

__declspec(dllexport) WatchView* 
WatchView::Instantiate(BMessage* archive)
{
	if (!validate_instantiation(archive, "WatchView")) 
	{
		return 0;
	}
	return new WatchView(archive);
}

// ===========================================================================
// Archive
// ===========================================================================

status_t 
WatchView::Archive(BMessage* archive, bool deep) const
{
	// Tell the overridden BView to archive itself.
	BView::Archive(archive, deep);
  
	// Store our signature and the name of our class in the archive.
	archive->AddString("add_on", APP_SIGNATURE);
	archive->AddString("class", "WatchView");

	return B_OK;
}  

// ===========================================================================
// Draw
// ===========================================================================

void 
WatchView::Draw(BRect updateRect)
{
	// Create the string.
	char string[5];
	sprintf(string, "@%03d", GetInternetTime());
	
	// Center the string horizontally and vertically.
	font_height height;
	GetFontHeight(&height);
	BRect rect = Bounds();
	float width = StringWidth(string);
	float x = 1 + (rect.Width() - width)/2;
	float y = 1 + height.ascent
			    + (rect.Height() - (height.ascent + height.descent))/2;

	// Draw the string. 
	SetLowColor(COLOR_BACKGROUND);
	SetHighColor(COLOR_FOREGROUND);
	SetDrawingMode(B_OP_OVER);
	DrawString(string, BPoint(x, y));
}

// ===========================================================================
// MouseDown
// ===========================================================================

void
WatchView::MouseDown(BPoint point) 
{
	// Create a pop-up menu object that is not in radio mode.
	BPopUpMenu* menu = new BPopUpMenu("WatchView", false, false);
	BMenuItem* item; 

	// Add the "About" item.
	item = new BMenuItem("About" B_UTF8_ELLIPSIS, 
		new BMessage(B_ABOUT_REQUESTED));

	menu->AddItem(item);

	// Add the "Quit" item.
	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED));
	menu->AddItem(item);

	// The messages of the menu items should be sent to ourselves.
	menu->SetTargetForItems(this);

	// Pop up the menu and wait for the user to make a selection.
	ConvertToScreen(&point); 
	menu->Go(point, true, false, true); 
	delete menu;
}

// ===========================================================================
// MessageReceived
// ===========================================================================

void
WatchView::MessageReceived(BMessage* message)
{
	// Figure out what kind of message was received.
	switch(message->what) 
	{
		// The "About" item.
		case B_ABOUT_REQUESTED: 
		{
			(new BAlert("About WebWatch", 
				"WebWatch -- A port of Swatch Internet Time\n\n"
				"Version " VERSION_STRING "\n"
				"Released on " VERSION_DATE_STRING "\n\n"
				"Created by Matthijs Hollemans\n"
				"matthijs@weirdexperiments.com\n"
				"http://www.weirdexperiments.com\n\n"
				"WebWatch is public domain. No warranties expressed"
				" or implied.",
				"OK"))->Go(0);
	
			break;
		}

		// The "Quit" item.
		case B_QUIT_REQUESTED:
		{
			// According to the Be Book, we are not allowed to do this
			// since we're a view that's sitting on the Deskbar's shelf.
			// But it works just fine for me, and I see no other way to
			// make a Deskbar replicant quit itself.
			BDeskbar deskbar;
			deskbar.RemoveItem("WebWatch");
			break;
		}

		// Let the original BView handle the message.
		default: 
		{
			BView::MessageReceived(message);
			break;
		}
	}
}

// ===========================================================================
// Pulse
// ===========================================================================

void 
WatchView::Pulse(void)
{
	int32 currentInternetTime = GetInternetTime();
	if (oldInternetTime != currentInternetTime)
	{
		oldInternetTime = currentInternetTime;
		Invalidate();
	}
}

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ===========================================================================
// GetInternetTime
// ===========================================================================

int32
WatchView::GetInternetTime(void)
{
	// First we get the current time as the number of seconds since 1 January
	// 1970 and add an hour's worth of seconds to adjust for the Biel Mean
	// Time (BMT). Then we get the number of seconds that have passed today,
	// and divide it with the length of a beat to get the number of elapsed
	// beats.
	return (int32) (((real_time_clock() + 3600) % 86400) / 86.4);
}

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
